#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Add container number info to compute node

Revision ID: 4bf34495d060
Revises: 53a8b515057e
Create Date: 2017-04-17 10:06:17.340957

"""

# revision identifiers, used by Alembic.
revision = '4bf34495d060'
down_revision = '53a8b515057e'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('compute_node',
                  sa.Column('total_containers', sa.Integer(), nullable=False))
    op.add_column('compute_node',
                  sa.Column('running_containers',
                            sa.Integer(), nullable=False))
    op.add_column('compute_node',
                  sa.Column('paused_containers', sa.Integer(), nullable=False))
    op.add_column('compute_node',
                  sa.Column('stopped_containers',
                            sa.Integer(), nullable=False))
